var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { isStateNode } from "./Node";
import mitt from "./Mitt";
import { generateUUID, generateTimeStamp } from "./Utils";
var ProvenanceGraph = /** @class */ (function () {
    function ProvenanceGraph(application, userid, rootNode) {
        if (userid === void 0) { userid = "Unknown"; }
        this._nodes = {};
        this._mitt = mitt();
        this.application = application;
        if (rootNode) {
            this.root = rootNode;
        }
        else {
            this.root = {
                id: generateUUID(),
                label: "Root",
                metadata: {
                    createdBy: userid,
                    createdOn: generateTimeStamp()
                },
                children: [],
                artifacts: {}
            };
        }
        this.addNode(this.root);
        this._current = this.root;
    }
    ProvenanceGraph.prototype.addNode = function (node) {
        if (this._nodes[node.id]) {
            throw new Error("Node already added!");
        }
        this._nodes[node.id] = node;
        this._mitt.emit("nodeAdded", node);
    };
    ProvenanceGraph.prototype.getNode = function (id) {
        var result = this._nodes[id];
        if (!result) {
            throw new Error("Node not found!");
        }
        return this._nodes[id];
    };
    Object.defineProperty(ProvenanceGraph.prototype, "current", {
        get: function () {
            return this._current;
        },
        set: function (node) {
            if (!this._nodes[node.id]) {
                throw new Error("Node not found!");
            }
            this._current = node;
            this._mitt("currentChanged", node);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ProvenanceGraph.prototype, "nodes", {
        get: function () {
            return this._nodes;
        },
        enumerable: true,
        configurable: true
    });
    ProvenanceGraph.prototype.emitNodeChangedEvent = function (node) {
        if (!this._nodes[node.id]) {
            throw new Error("Node not found");
        }
        this._mitt.emit("nodeChanged", node);
    };
    ProvenanceGraph.prototype.on = function (type, handler) {
        this._mitt.on(type, handler);
    };
    ProvenanceGraph.prototype.off = function (type, handler) {
        this._mitt.off(type, handler);
    };
    ProvenanceGraph.restoreProvenanceGraph = function (serializedGraph) {
        var nodes = {};
        // ! Explore
        for (var _i = 0, _a = serializedGraph.nodes; _i < _a.length; _i++) {
            var node = _a[_i];
            nodes[node.id] = __assign({}, node);
        }
        for (var _b = 0, _c = Object.keys(nodes); _b < _c.length; _b++) {
            var nodeId = _c[_b];
            var node = nodes[nodeId];
            node.children = node.children.map(function (id) { return nodes[id]; });
            if ("parent" in node) {
                node.parent = nodes[node.parent];
            }
        }
        var graph = new ProvenanceGraph(serializedGraph.application);
        graph._nodes = nodes;
        graph._current = nodes[serializedGraph.current];
        graph.root = nodes[serializedGraph.root];
        return graph;
    };
    ProvenanceGraph.serializeProvenanceGraph = function (graph) {
        var nodes = Object.keys(graph.nodes).map(function (nodeId) {
            var node = graph.getNode(nodeId);
            var serializedNode = __assign({}, node);
            if (isStateNode(node)) {
                serializedNode.parent = node.parent.id;
            }
            serializedNode.children = node.children.map(function (child) { return child.id; });
            return serializedNode;
        });
        return {
            nodes: nodes,
            root: graph.root.id,
            application: graph.application,
            current: graph.current.id
        };
    };
    return ProvenanceGraph;
}());
export { ProvenanceGraph };
//# sourceMappingURL=ProvenanceGraph.js.map