import { Handler } from "./Provenance/Handler";
import { generateFunctionUUID } from ".";
import { IProvenanceGraph } from "./Provenance/IProvenanceGraph";
import { ProvenanceGraphTraverser } from "./Provenance/ProvenanceGraphTraverser";
import { ProvenanceTracker } from "./Provenance/ProvenanceTracker";
import { IProvenanceGraphTraverser } from "./Provenance/IProvenanceGraphTraverser";
import { IProvenanceTracker } from "./Provenance/IProvenanceTracker";
import { IActionFunctionRegistry } from "./Provenance/IActionFunctionRegistry";
import { Application } from "./Application";
import { Mitt } from "./Provenance/Mitt";
import { IView } from "./IView";
import { IViewModel } from "./IViewModel";
import { apply } from "./Command/Command";
export class ViewModelBase implements IViewModel {
  App: Application;
  tracker: IProvenanceTracker;
  traverser: IProvenanceGraphTraverser;
  _uuid: string;
  comm: Mitt;

  get registry(): IActionFunctionRegistry {
    return this.App.registry;
  }

  get graph(): IProvenanceGraph {
    return this.App.graph;
  }

  constructor(view: IView, app: Application) {
    this.comm = view.comm;
    this._uuid = generateFunctionUUID();
    this.App = app;
    this.tracker = new ProvenanceTracker(this.registry, app.graph);
    this.traverser = new ProvenanceGraphTraverser(this.registry, app.graph);
    this.comm.emit("create");
    this.App.on("graph-loaded", this.load, this);
  }

  apply(args: any) {
    apply.call(this, ...args);
  }

  protected registerFunctions(
    str: string,
    func: Handler,
    thisArg: any,
    _do: boolean = true
  ) {
    if (_do) this.App.registry.register(`do_${str}`, func, thisArg);
    else this.App.registry.register(`undo_${str}`, func, thisArg);
  }

  protected load(graph: IProvenanceGraph) {
    this.App.graph = graph;
    this.tracker = new ProvenanceTracker(this.registry, this.App.graph);
    this.traverser = new ProvenanceGraphTraverser(
      this.registry,
      this.App.graph
    );
  }
}
